@@ -16,14 +16,15 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
+ * Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 #include <math.h>
+#include <libgitg/gitg-lane.h>
+#include <libgitg/gitg-revision.h>
+
 #include "gitg-cell-renderer-path.h"
-#include "gitg-lane.h"
-#include "gitg-utils.h"
 #include "gitg-label-renderer.h"
 
 #define GITG_CELL_RENDERER_PATH_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GITG_TYPE_CELL_RENDERER_PATH, GitgCellRendererPathPrivate))
@@ -37,7 +38,7 @@
 enum
 {
 	PROP_0,
-	
+
 	PROP_REVISION,
 	PROP_NEXT_REVISION,
 	PROP_LANE_WIDTH,
@@ -54,7 +55,7 @@ struct _GitgCellRendererPathPrivate
 	guint lane_width;
 	guint triangle_width;
 	guint dot_width;
-	
+
 	gint last_height;
 };
 
@@ -87,12 +88,12 @@ total_width(GitgCellRendererPath *self, GtkWidget *widget)
 {
 	PangoFontDescription *font;
 	g_object_get(self, "font-desc", &font, NULL);
-	
+
 	gint offset = 0;
-	
+
 	if (is_dummy(self->priv->revision))
 		offset = self->priv->lane_width;
-	
+
 	return num_lanes(self) * self->priv->lane_width + 
 	       gitg_label_renderer_width(widget, font, self->priv->labels) +
 	       offset;
@@ -102,58 +103,58 @@ static void
 gitg_cell_renderer_path_finalize(GObject *object)
 {
 	GitgCellRendererPath *self = GITG_CELL_RENDERER_PATH(object);
-	
+
 	gitg_revision_unref(self->priv->revision);
 	gitg_revision_unref(self->priv->next_revision);
-	
+
 	g_slist_free(self->priv->labels);
 
 	G_OBJECT_CLASS(gitg_cell_renderer_path_parent_class)->finalize(object);
 }
 
 static void
-renderer_get_size(GtkCellRenderer *renderer, GtkWidget *widget, GdkRectangle *area, gint *xoffset, gint *yoffset, gint *width, gint *height)
+renderer_get_size(GtkCellRenderer *renderer, GtkWidget *widget, const GdkRectangle *area, gint *xoffset, gint *yoffset, gint *width, gint *height)
 {
 	GitgCellRendererPath *self = GITG_CELL_RENDERER_PATH(renderer);
 
 	if (xoffset)
 		*xoffset = 0;
-	
+
 	if (yoffset)
 		*yoffset = 0;
-	
+
 	if (width)
 		*width = total_width(self, widget);
-	
+
 	if (height)
 		*height = area ? area->height : 1;
 }
 
 static void
-draw_arrow(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area, gint8 laneidx, gboolean top)
+draw_arrow(GitgCellRendererPath *self, cairo_t *cr, const GdkRectangle *area, gint8 laneidx, gboolean top)
 {
 	gdouble cw = self->priv->lane_width;
 	gdouble xpos = area->x + laneidx * cw + cw / 2.0;
 	gdouble df = (top ? -1 : 1) * 0.25 * area->height;
 	gdouble ypos = area->y + area->height / 2.0 + df;
 	gdouble q = cw / 4.0;
-	
+
 	cairo_move_to(cr, xpos - q, ypos + (top ? q : -q));
 	cairo_line_to(cr, xpos, ypos);
 	cairo_line_to(cr, xpos + q, ypos + (top ? q : -q));
 	cairo_stroke(cr);
-	
+
 	cairo_move_to(cr, xpos, ypos);
 	cairo_line_to(cr, xpos, ypos - df);
 	cairo_stroke(cr);
-	
+
 	//cairo_move_to(cr, xpos, ypos);
 	//cairo_line_to(cr, xpos, ypos + (top ? 1 : -1) * area->height / 2.0);
 	//cairo_stroke(cr);
 }
 
 static void
-draw_paths_real(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area, GitgRevision *revision, gdouble yoffset)
+draw_paths_real(GitgCellRendererPath *self, cairo_t *cr, const GdkRectangle *area, GitgRevision *revision, gdouble yoffset)
 {
 	if (!revision)
 		return;
@@ -163,23 +164,23 @@ draw_paths_real(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area, Git
 	gdouble cw = self->priv->lane_width;
 	gdouble ch = area->height / 2.0;
 	GitgLane *lane;
-	
+
 	while (lanes)
 	{
 		GSList *item;
 
 		lane = (GitgLane *)(lanes->data);
 		gitg_color_set_cairo_source(lane->color, cr);
-		
+
 		for (item = lane->from; item; item = item->next)
 		{
 			gint8 from = (gint8)GPOINTER_TO_INT(item->data);
-			
+
 			cairo_move_to(cr, area->x + from * cw + cw / 2.0, area->y + yoffset * ch);
 			cairo_curve_to(cr, area->x + from * cw + cw / 2.0, area->y + (yoffset + 1) * ch,
 						   area->x + to * cw + cw / 2.0, area->y + (yoffset + 1) * ch,
 						   area->x + to * cw + cw / 2.0, area->y + (yoffset + 2) * ch);
-			
+
 			cairo_stroke(cr);
 		}
 
@@ -189,39 +190,39 @@ draw_paths_real(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area, Git
 }
 
 static void
-draw_top_paths(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area)
+draw_top_paths(GitgCellRendererPath *self, cairo_t *cr, const GdkRectangle *area)
 {
 	draw_paths_real(self, cr, area, self->priv->revision, -1);
 }
 
 static void
-draw_bottom_paths(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area)
+draw_bottom_paths(GitgCellRendererPath *self, cairo_t *cr, const GdkRectangle *area)
 {
 	draw_paths_real(self, cr, area, self->priv->next_revision, 1);
 }
 
 static void
-draw_arrows(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area)
+draw_arrows(GitgCellRendererPath *self, cairo_t *cr, const GdkRectangle *area)
 {
 	GSList *item;
 	gint8 to = 0;
-	
+
 	for (item = gitg_revision_get_lanes(self->priv->revision); item; item = item->next)
 	{
 		GitgLane *lane = (GitgLane *)item->data;
 		gitg_color_set_cairo_source(lane->color, cr);
-		
+
 		if (lane->type & GITG_LANE_TYPE_START)
 			draw_arrow(self, cr, area, to, TRUE);
 		else if (lane->type & GITG_LANE_TYPE_END)
 			draw_arrow(self, cr, area, to, FALSE);
-		
+
 		++to;
 	}
 }
 
 static void
-draw_paths(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area)
+draw_paths(GitgCellRendererPath *self, cairo_t *cr, const GdkRectangle *area)
 {
 	cairo_set_line_width(cr, 2);
 	//cairo_set_source_rgb(cr, 0.45, 0.6, 0.74);
@@ -233,28 +234,28 @@ draw_paths(GitgCellRendererPath *self, cairo_t *cr, GdkRectangle *area)
 }
 
 static void
-draw_labels(GitgCellRendererPath *self, GtkWidget *widget, cairo_t *context, GdkRectangle *area)
+draw_labels(GitgCellRendererPath *self, GtkWidget *widget, cairo_t *context, const GdkRectangle *area)
 {
 	gint offset = num_lanes(self) * self->priv->lane_width;
 	PangoFontDescription *font;
-	
+
 	if (is_dummy(self->priv->revision))
 		offset += self->priv->lane_width;
-	
+
 	g_object_get(self, "font-desc", &font, NULL);
-	
+
 	cairo_translate(context, offset, 0.0);
 	gitg_label_renderer_draw(widget, font, context, self->priv->labels, area);
 }
 
 static void
-draw_indicator_triangle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *context, GdkRectangle *area)
+draw_indicator_triangle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *context, const GdkRectangle *area)
 {
 	gdouble offset = gitg_revision_get_mylane(self->priv->revision) * self->priv->lane_width + (self->priv->lane_width - self->priv->triangle_width) / 2.0;
 	gdouble radius = self->priv->triangle_width / 2.0;
 	gdouble xs;
 	int xd;
-	
+
 	if (lane->type & GITG_LANE_SIGN_LEFT)
 	{
 		xs = radius;
@@ -265,13 +266,13 @@ draw_indicator_triangle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *con
 		xs = -radius;
 		xd = 1;
 	}
-	
+
 	cairo_set_line_width(context, 2.0);
 	cairo_move_to(context, area->x + offset + radius + xs, area->y + (area->height - self->priv->triangle_width) / 2);
 	cairo_rel_line_to(context, 0, self->priv->triangle_width);
 	cairo_rel_line_to(context, xd * self->priv->triangle_width, -self->priv->triangle_width / 2);
 	cairo_close_path(context);
-	
+
 	cairo_set_source_rgb(context, 0, 0, 0);
 	cairo_stroke_preserve(context);
 
@@ -280,18 +281,18 @@ draw_indicator_triangle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *con
 }
 
 static void
-draw_indicator_circle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *context, GdkRectangle *area)
+draw_indicator_circle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *context, const GdkRectangle *area)
 {
 	gdouble offset = gitg_revision_get_mylane(self->priv->revision) * self->priv->lane_width + (self->priv->lane_width - self->priv->dot_width) / 2.0;
 	gdouble radius = self->priv->dot_width / 2.0;
-	
+
 	if (is_dummy(self->priv->revision))
 		offset += self->priv->lane_width;
 
 	cairo_set_line_width(context, 2.0);
 	cairo_arc(context, area->x + offset + radius, area->y + area->height / 2.0, radius, 0, 2 * M_PI);
 	cairo_set_source_rgb(context, 0, 0, 0);
-	
+
 	if (is_dummy(self->priv->revision))
 	{
 		cairo_stroke(context);
@@ -300,16 +301,16 @@ draw_indicator_circle(GitgCellRendererPath *self, GitgLane *lane, cairo_t *conte
 	{
 		cairo_stroke_preserve(context);
 		gitg_color_set_cairo_source(lane->color, context);
-	
+
 		cairo_fill(context);
 	}
 }
 
 static void
-draw_indicator(GitgCellRendererPath *self, cairo_t *context, GdkRectangle *area)
+draw_indicator(GitgCellRendererPath *self, cairo_t *context, const GdkRectangle *area)
 {
 	GitgLane *lane = gitg_revision_get_lane(self->priv->revision);
-	
+
 	if (lane->type & GITG_LANE_SIGN_LEFT || lane->type & GITG_LANE_SIGN_RIGHT)
 		draw_indicator_triangle(self, lane, context, area);
 	else
@@ -317,31 +318,28 @@ draw_indicator(GitgCellRendererPath *self, cairo_t *context, GdkRectangle *area)
 }
 
 static void
-renderer_render(GtkCellRenderer *renderer, GdkDrawable *window, GtkWidget *widget, GdkRectangle *area, GdkRectangle *cell_area, GdkRectangle *expose_area, GtkCellRendererState flags)
+renderer_render(GtkCellRenderer *renderer, cairo_t *cr, GtkWidget *widget, const GdkRectangle *area, const GdkRectangle *cell_area, GtkCellRendererState flags)
 {
 	GitgCellRendererPath *self = GITG_CELL_RENDERER_PATH(renderer);
-	
+
 	self->priv->last_height = area->height;
 
-	cairo_t *cr = gdk_cairo_create(window);
-	
-	cairo_rectangle(cr, area->x, area->y, area->width, area->height);
+	gdk_cairo_rectangle (cr, area);
 	cairo_clip(cr);
-	
+
 	draw_paths(self, cr, area);
-	
+
 	/* draw indicator */
 	draw_indicator(self, cr, area);
-	
+
 	/* draw labels */
 	draw_labels(self, widget, cr, area);
-	cairo_destroy(cr);
-	
-	area->x += total_width(self, widget);
-	cell_area->x += total_width(self, widget);
+
+	((GdkRectangle *) area)->x += total_width(self, widget);
+	((GdkRectangle *) cell_area)->x += total_width(self, widget);
 
 	if (GTK_CELL_RENDERER_CLASS(parent_class)->render)
-		GTK_CELL_RENDERER_CLASS(parent_class)->render(renderer, window, widget, area, cell_area, expose_area, flags);
+		GTK_CELL_RENDERER_CLASS(parent_class)->render(renderer, cr, widget, area, cell_area, flags);
 }
 
 static void
@@ -379,7 +377,7 @@ static void
 gitg_cell_renderer_path_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	GitgCellRendererPath *self = GITG_CELL_RENDERER_PATH(object);
-	
+
 	switch (prop_id)
 	{
 		case PROP_REVISION:
@@ -414,11 +412,11 @@ gitg_cell_renderer_path_class_init(GitgCellRendererPathClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	GtkCellRendererClass *renderer_class = GTK_CELL_RENDERER_CLASS(klass);
-	
+
 	object_class->finalize = gitg_cell_renderer_path_finalize;
 	object_class->get_property = gitg_cell_renderer_path_get_property;
 	object_class->set_property = gitg_cell_renderer_path_set_property;
-	
+
 	renderer_class->get_size = renderer_get_size;
 	renderer_class->render = renderer_render;
 
@@ -436,7 +434,7 @@ gitg_cell_renderer_path_class_init(GitgCellRendererPathClass *klass)
 							      "NEXT_REVISION",
 							      "The next revision",
 							      GITG_TYPE_REVISION,
-							      G_PARAM_READWRITE));	
+							      G_PARAM_READWRITE));
 
 	g_object_class_install_property(object_class, PROP_LANE_WIDTH,
 					 g_param_spec_uint("lane-width",
@@ -478,7 +476,7 @@ static void
 gitg_cell_renderer_path_init(GitgCellRendererPath *self)
 {
 	self->priv = GITG_CELL_RENDERER_PATH_GET_PRIVATE(self);
-	
+
 	self->priv->lane_width = DEFAULT_LANE_WIDTH;
 	self->priv->dot_width = DEFAULT_DOT_WIDTH;
 	self->priv->triangle_width = DEFAULT_TRIANGLE_WIDTH;
@@ -495,15 +493,15 @@ gitg_cell_renderer_path_get_ref_at_pos (GtkWidget *widget, GitgCellRendererPath
 {
 	g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 	g_return_val_if_fail (GITG_IS_CELL_RENDERER_PATH (renderer), NULL);
-	
+
 	PangoFontDescription *font;
 	g_object_get (renderer, "font-desc", &font, NULL);
-	
+
 	gint offset = 0;
-	
+
 	if (is_dummy(renderer->priv->revision))
 		offset = renderer->priv->lane_width;
-	
+
 	x -= num_lanes(renderer) * renderer->priv->lane_width + offset;
 
 	return gitg_label_renderer_get_ref_at_pos (widget, font, renderer->priv->labels, x, hot_x);
@@ -517,6 +515,6 @@ gitg_cell_renderer_path_render_ref (GtkWidget *widget, GitgCellRendererPath *ren
 
 	PangoFontDescription *font;
 	g_object_get(renderer, "font-desc", &font, NULL);
-	
+
 	return gitg_label_renderer_render_ref (widget, font, ref, renderer->priv->last_height, minwidth);
 }
