@@ -54,6 +54,13 @@ typedef enum
 	GITG_DIFF_ITER_TYPE_HUNK
 } GitgDiffIterType;
 
+typedef enum
+{
+	GITG_DIFF_LINE_TYPE_NONE,
+	GITG_DIFF_LINE_TYPE_ADD,
+	GITG_DIFF_LINE_TYPE_REMOVE,
+} GitgDiffLineType;
+
 struct _GitgDiffView
 {
 	GtkSourceView parent;
@@ -69,6 +76,10 @@ struct _GitgDiffViewClass
 	void (*hunk_added)(GitgDiffView *view, GitgDiffIter *iter);
 };
 
+typedef gchar *(*GitgDiffViewLabelFunc) (GitgDiffView *view,
+                                         gint          line,
+                                         gpointer      user_data);
+
 GType gitg_diff_view_get_type(void) G_GNUC_CONST;
 GitgDiffView *gitg_diff_view_new(void);
 
@@ -86,6 +97,19 @@ GitgDiffIterType gitg_diff_iter_get_type(GitgDiffIter *iter);
 void gitg_diff_iter_set_visible(GitgDiffIter *iter, gboolean visible);
 gboolean gitg_diff_iter_get_index(GitgDiffIter *iter, gchar **from, gchar **to);
 
+gboolean gitg_diff_view_get_header_at_iter (GitgDiffView *view, GtkTextIter const *iter, GitgDiffIter *diff_iter);
+gboolean gitg_diff_view_get_hunk_at_iter (GitgDiffView *view, GtkTextIter const *iter, GitgDiffIter *diff_iter);
+
+void gitg_diff_iter_get_bounds (GitgDiffIter const *iter, GtkTextIter *start, GtkTextIter *end);
+
+GitgDiffLineType gitg_diff_view_get_line_type (GitgDiffView *view, GtkTextIter const *iter);
+void gitg_diff_view_clear_line (GitgDiffView *view, GtkTextIter const *iter, GitgDiffLineType old_type, GitgDiffLineType new_type);
+
+void gitg_diff_view_set_label_func (GitgDiffView *view,
+                                    GitgDiffViewLabelFunc func,
+                                    gpointer user_data,
+                                    GDestroyNotify destroy_notify);
+
 G_END_DECLS
 
 #endif /* __GITG_DIFF_VIEW_H__ */
